@@ -26,6 +26,7 @@
 
 #include <stdint.h>
 #include <errno.h>
+#include <stdbool.h>
 #include <string.h>
 #include <sys/socket.h>
 
@@ -164,7 +165,7 @@ static int t4_cookie_release(int err, void *data)
 
 	DBG("%p", cookie);
 
-	if (cookie == NULL)
+	if (!cookie)
 		return err;
 
 	if (err < 0 && cookie->cb)
@@ -186,7 +187,7 @@ static int ISO_send_cmd(uint8_t class,
 			uint8_t param2,
 			uint8_t *cmd_data,
 			uint8_t cmd_data_length,
-			near_bool_t le,
+			bool le,
 			near_recv cb,
 			void *in_data)
 {
@@ -198,7 +199,7 @@ static int ISO_send_cmd(uint8_t class,
 	DBG("CLA-%02x INS-%02x P1-%02x P2-%02x",
 			class, instruction, param1, param2);
 
-	if (le == FALSE) {
+	if (!le) {
 		if (cmd_data)
 			total_cmd_length = APDU_HEADER_LEN + cmd_data_length;
 		else
@@ -208,7 +209,7 @@ static int ISO_send_cmd(uint8_t class,
 	}
 
 	cmd = g_try_malloc0(total_cmd_length);
-	if (cmd == NULL) {
+	if (!cmd) {
 		DBG("Mem alloc failed");
 		err = -ENOMEM;
 		goto out_err;
@@ -224,7 +225,7 @@ static int ISO_send_cmd(uint8_t class,
 		memcpy(cmd->data, cmd_data, cmd_data_length);
 		/* The Le byte set to 0x00 defined that any length
 		 * of PICC response is allowed */
-		if (le == TRUE)
+		if (le)
 			cmd->data[cmd_data_length] = 0;
 	}
 
@@ -255,7 +256,7 @@ static int ISO_Select(uint8_t *filename, uint8_t fnamelen, uint8_t P1,
 			0x00,		/* P2: First or only occurrence */
 			filename,	/* cmd_data */
 			fnamelen,	/* uint8_t cmd_data_length*/
-			FALSE,
+			false,
 			cb,
 			cookie);
 }
@@ -272,7 +273,7 @@ static int ISO_ReadBinary(uint16_t offset, uint8_t readsize,
 			(uint8_t) (offset & 0xFF),
 			0,		/* no data send */
 			readsize,	/* bytes to read */
-			FALSE,
+			false,
 			cb,
 			cookie);
 }
@@ -289,7 +290,7 @@ static int ISO_Update(uint16_t offset, uint8_t nlen,
 			(uint8_t) (offset & 0xFF),
 			data,			/* length of NDEF data */
 			nlen,			/* NLEN + NDEF data */
-			FALSE,
+			false,
 			cb,
 			cookie);
 }
@@ -370,7 +371,7 @@ static int t4_readbin_NDEF_ID(uint8_t *resp, int length, void *data)
 		return t4_cookie_release(err, cookie);
 
 	tag = near_tag_get_tag(cookie->adapter_idx, cookie->target_idx);
-	if (tag == NULL)
+	if (!tag)
 		return t4_cookie_release(-ENOMEM, cookie);
 
 	near_tag_set_max_ndef_size(tag, cookie->max_ndef_size);
@@ -420,7 +421,7 @@ static int t4_select_NDEF_ID(uint8_t *resp, int length, void *data)
 static int t4_readbin_cc(uint8_t *resp, int length, void *data)
 {
 	struct t4_cookie *cookie = data;
-	struct type4_cc *read_cc;
+	struct type4_cc *read_cc = (struct type4_cc *)&resp[1];
 
 	DBG("%d", length);
 
@@ -434,16 +435,6 @@ static int t4_readbin_cc(uint8_t *resp, int length, void *data)
 		return t4_cookie_release(-EIO, cookie);
 	}
 
-	/* -2 for status word and -1 is for NFC first byte... */
-	read_cc = g_try_malloc0(length - 2 - NFC_STATUS_BYTE_LEN);
-	if (read_cc == NULL) {
-		DBG("Mem alloc failed");
-
-		return t4_cookie_release(-ENOMEM, cookie);
-	}
-
-	memcpy(read_cc, &resp[1], length - 2 - NFC_STATUS_BYTE_LEN) ;
-
 	cookie->r_apdu_max_size = g_ntohs(read_cc->max_R_apdu_data_size) -
 			APDU_HEADER_LEN;
 	cookie->c_apdu_max_size = g_ntohs(read_cc->max_C_apdu_data_size);
@@ -486,7 +477,7 @@ static int t4_select_cc(uint8_t *resp, int length, void *data)
 
 		cookie->tag = near_tag_get_tag(cookie->adapter_idx,
 						cookie->target_idx);
-		if (cookie->tag == NULL)
+		if (!cookie->tag)
 			return t4_cookie_release(-ENOMEM, cookie);
 
 		near_tag_set_blank(cookie->tag, TRUE);
@@ -563,7 +554,7 @@ static int nfctype4_read(uint32_t adapter_idx,
 	DBG("");
 
 	cookie = g_try_malloc0(sizeof(struct t4_cookie));
-	if (cookie == NULL)
+	if (!cookie)
 		return -ENOMEM;
 
 	cookie->adapter_idx = adapter_idx;
@@ -596,7 +587,6 @@ static int data_write_cb(uint8_t *resp, int length, void *data)
 
 	if (cookie->ndef->offset >= cookie->ndef->length) {
 		DBG("Done writing");
-		near_adapter_disconnect(cookie->adapter_idx);
 
 		if (cookie->cb)
 			cookie->cb(cookie->adapter_idx, cookie->target_idx, 0);
@@ -634,10 +624,10 @@ static int data_write(uint32_t adapter_idx, uint32_t target_idx,
 
 	cookie = g_try_malloc0(sizeof(struct t4_cookie));
 
-	if (cookie == NULL) {
+	if (!cookie) {
 		err = -ENOMEM;
 
-		if (cb != NULL)
+		if (cb)
 			cb(adapter_idx, target_idx, err);
 
 		return err;
@@ -690,13 +680,13 @@ static int nfctype4_write(uint32_t adapter_idx, uint32_t target_idx,
 
 	DBG("");
 
-	if (ndef == NULL || cb == NULL) {
+	if (!ndef || !cb) {
 		err = -EINVAL;
 		goto out_err;
 	}
 
 	tag = near_tag_get_tag(adapter_idx, target_idx);
-	if (tag == NULL) {
+	if (!tag) {
 		err = -EINVAL;
 		goto out_err;
 	}
@@ -704,7 +694,7 @@ static int nfctype4_write(uint32_t adapter_idx, uint32_t target_idx,
 	err = data_write(adapter_idx, target_idx, ndef, tag, cb);
 
 out_err:
-	if (cb != NULL)
+	if (cb)
 		cb(adapter_idx, target_idx, err);
 
 	return err;
@@ -735,7 +725,7 @@ static int nfctype4_check_presence(uint32_t adapter_idx,
 	DBG("");
 
 	cookie = g_try_malloc0(sizeof(struct t4_cookie));
-	if (cookie == NULL)
+	if (!cookie)
 		return -ENOMEM;
 
 	cookie->adapter_idx = adapter_idx;
@@ -797,7 +787,7 @@ static int read_cc_file(uint8_t *resp, int length, void *data)
 
 	/* -2 for status word and -1 is for NFC first byte... */
 	read_cc = g_try_malloc0(length - 2 - NFC_STATUS_BYTE_LEN);
-	if (read_cc == NULL) {
+	if (!read_cc) {
 		err = -ENOMEM;
 		goto out_err;
 	}
@@ -807,7 +797,7 @@ static int read_cc_file(uint8_t *resp, int length, void *data)
 	cookie->max_ndef_size = g_ntohs(read_cc->tlv_fc.max_ndef_size);
 
 	tag = near_tag_get_tag(cookie->adapter_idx, cookie->target_idx);
-	if (tag == NULL) {
+	if (!tag) {
 		err = -EINVAL;
 		goto out_err;
 	}
@@ -919,7 +909,7 @@ static int format_resp(uint8_t *resp, int length, void *data)
 	}
 
 	tag = near_tag_get_tag(cookie->adapter_idx, cookie->target_idx);
-	if (tag == NULL)
+	if (!tag)
 		return t4_cookie_release(-EINVAL, cookie);
 
 	DBG("Formatting is done");
@@ -977,7 +967,7 @@ static int write_data_to_ndef_file(uint8_t *resp, int length, void *data)
 				+ ARRAY_SIZE(ndef_nlen);
 
 	cmd_data = g_try_malloc0(cmd_data_length);
-	if (cmd_data == NULL) {
+	if (!cmd_data) {
 		err = -ENOMEM;
 		goto out_err;
 	}
@@ -990,7 +980,7 @@ static int write_data_to_ndef_file(uint8_t *resp, int length, void *data)
 
 	err = ISO_send_cmd(PICC_CLASS, WRITE_DATA_TO_FILE,
 				0x00, 0x00, cmd_data, cmd_data_length,
-				TRUE, format_resp, cookie);
+				true, format_resp, cookie);
 	if (err < 0) {
 		near_error("wrtie data to ndef file req failed %d", err);
 		goto out_err;
@@ -1028,7 +1018,7 @@ static int create_ndef_file(uint8_t *resp, int length, void *data)
 	}
 
 	ndef = g_try_malloc0(sizeof(struct desfire_std_file));
-	if (ndef == NULL) {
+	if (!ndef) {
 		err = -ENOMEM;
 		goto out_err;
 	}
@@ -1045,7 +1035,7 @@ static int create_ndef_file(uint8_t *resp, int length, void *data)
 	err = ISO_send_cmd(PICC_CLASS, CREATE_STD_DATA_FILE,
 				0x00, 0x00, (uint8_t *)ndef,
 				sizeof(struct desfire_std_file),
-				TRUE, write_data_to_ndef_file, cookie);
+				true, write_data_to_ndef_file, cookie);
 	if (err < 0) {
 		near_error("create ndef file req failed %d", err);
 		goto out_err;
@@ -1089,7 +1079,7 @@ static int write_data_to_cc_file(uint8_t *resp, int length, void *data)
 	}
 
 	cc = g_try_malloc0(sizeof(struct desfire_cc_file));
-	if (cc == NULL) {
+	if (!cc) {
 		err = -ENOMEM;
 		goto out_err;
 	}
@@ -1110,7 +1100,7 @@ static int write_data_to_cc_file(uint8_t *resp, int length, void *data)
 	err = ISO_send_cmd(PICC_CLASS, WRITE_DATA_TO_FILE,
 				0x00, 0x00, (uint8_t *)cc,
 				sizeof(struct desfire_cc_file),
-				TRUE, create_ndef_file, cookie);
+				true, create_ndef_file, cookie);
 	if (err < 0) {
 		near_error("write data to cc file req failed %d", err);
 		goto out_err;
@@ -1149,7 +1139,7 @@ static int create_cc_file(uint8_t *resp, int length, void *data)
 	}
 
 	cc = g_try_malloc0(sizeof(struct desfire_std_file));
-	if (cc == NULL) {
+	if (!cc) {
 		err = -ENOMEM;
 		goto out_err;
 	}
@@ -1165,7 +1155,7 @@ static int create_cc_file(uint8_t *resp, int length, void *data)
 				CREATE_STD_DATA_FILE,
 				0x00, 0x00, (uint8_t *)cc,
 				sizeof(struct desfire_std_file),
-				TRUE, write_data_to_cc_file, cookie);
+				true, write_data_to_cc_file, cookie);
 	if (err < 0) {
 		near_error("create cc file req failed %d", err);
 		goto out_err;
@@ -1205,7 +1195,7 @@ static int select_application_1(uint8_t *resp, int length, void *data)
 	/* Step4 : Select application (which is created just now) */
 	cmd_data_length = ARRAY_SIZE(desfire_aid_1);
 	cmd_data = g_try_malloc0(cmd_data_length);
-	if (cmd_data == NULL) {
+	if (!cmd_data) {
 		err = -ENOMEM;
 		goto out_err;
 	}
@@ -1213,7 +1203,7 @@ static int select_application_1(uint8_t *resp, int length, void *data)
 	memcpy(cmd_data, desfire_aid_1, cmd_data_length);
 	err = ISO_send_cmd(PICC_CLASS, SELECT_APPLICATION,
 				0x00, 0x00, cmd_data, cmd_data_length,
-				TRUE, create_cc_file, cookie);
+				true, create_cc_file, cookie);
 	if (err < 0) {
 		near_error("select application1 req failed %d", err);
 		goto out_err;
@@ -1251,7 +1241,7 @@ static int create_application(uint8_t *resp, int length, void *data)
 	}
 
 	app = g_try_malloc0(sizeof(struct desfire_app));
-	if (app == NULL) {
+	if (!app) {
 		err = -ENOMEM;
 		goto out_err;
 	}
@@ -1266,7 +1256,7 @@ static int create_application(uint8_t *resp, int length, void *data)
 	err = ISO_send_cmd(PICC_CLASS, CREATE_APPLICATION,
 				0x00, 0x00, (uint8_t *)app,
 				sizeof(struct desfire_app),
-				TRUE, select_application_1, cookie);
+				true, select_application_1, cookie);
 	if (err < 0) {
 		near_error("create application req failed %d", err);
 		goto out_err;
@@ -1306,7 +1296,7 @@ static int select_application(uint8_t *resp, int length, void *data)
 	/* AID : 000000h */
 	cmd_data_length = ARRAY_SIZE(desfire_aid);
 	cmd_data = g_try_malloc0(cmd_data_length);
-	if (cmd_data == NULL) {
+	if (!cmd_data) {
 		err = -ENOMEM;
 		goto out_err;
 	}
@@ -1316,7 +1306,7 @@ static int select_application(uint8_t *resp, int length, void *data)
 	err = ISO_send_cmd(PICC_CLASS,
 				SELECT_APPLICATION,
 				0x00, 0x00, cmd_data, cmd_data_length,
-				TRUE, create_application, cookie);
+				true, create_application, cookie);
 	if (err < 0) {
 		near_error("select application req failed %d", err);
 		goto out_err;
@@ -1348,7 +1338,7 @@ static int get_version_frame3(uint8_t *resp, int length, void *data)
 
 		err = ISO_send_cmd(PICC_CLASS,
 					GET_VERSION_FRAME_RESPONSE_BYTE,
-					0x00, 0x00, NULL, 0, FALSE,
+					0x00, 0x00, NULL, 0, false,
 					select_application, cookie);
 		if (err < 0) {
 			near_error("get version3 req failed %d", err);
@@ -1387,7 +1377,7 @@ static int get_version_frame2(uint8_t *resp, int length, void *data)
 		cookie->memory_size = (1 << (resp[6] / 2));
 		err = ISO_send_cmd(PICC_CLASS,
 					GET_VERSION_FRAME_RESPONSE_BYTE,
-					0x00, 0x00, NULL, 0, FALSE,
+					0x00, 0x00, NULL, 0, false,
 					get_version_frame3, cookie);
 		if (err < 0) {
 			near_error("get version2 req failed %d", err);
@@ -1434,7 +1424,7 @@ static int nfctype4_format(uint32_t adapter_idx, uint32_t target_idx,
 	DBG("");
 
 	cookie = g_try_malloc0(sizeof(struct t4_cookie));
-	if (cookie == NULL)
+	if (!cookie)
 		return -ENOMEM;
 
 	cookie->adapter_idx = adapter_idx;
@@ -1443,7 +1433,7 @@ static int nfctype4_format(uint32_t adapter_idx, uint32_t target_idx,
 
 	/* Step1 : Get Version */
 	err = ISO_send_cmd(PICC_CLASS, GET_VERSION,
-				0x00, 0x00, NULL, 0, FALSE,
+				0x00, 0x00, NULL, 0, false,
 				get_version_frame2, cookie);
 
 	if (err < 0) {
