@@ -55,7 +55,7 @@ struct near_tag {
 	size_t data_length;
 	uint8_t *data;
 
-	uint32_t n_records;
+	uint32_t next_record;
 	GList *records;
 	bool blank;
 
@@ -181,53 +181,68 @@ static const char *protocol_string(struct near_tag *tag)
 	return protocol;
 }
 
-static DBusMessage *get_properties(DBusConnection *conn,
-					DBusMessage *msg, void *data)
+static gboolean property_get_type(const GDBusPropertyTable *property,
+					DBusMessageIter *iter, void *user_data)
 {
-	struct near_tag *tag = data;
-	const char *protocol, *type;
-	DBusMessage *reply;
-	DBusMessageIter array, dict;
-	dbus_bool_t readonly;
-
-	DBG("conn %p", conn);
-
-	reply = dbus_message_new_method_return(msg);
-	if (!reply)
-		return NULL;
-
-	dbus_message_iter_init_append(reply, &array);
-
-	near_dbus_dict_open(&array, &dict);
+	struct near_tag *tag = user_data;
+	const char *type;
 
 	type = type_string(tag);
-	if (type)
-		near_dbus_dict_append_basic(&dict, "Type",
-					DBUS_TYPE_STRING, &type);
+	if (!type)
+		return FALSE;
 
-	protocol = protocol_string(tag);
-	if (protocol)
-		near_dbus_dict_append_basic(&dict, "Protocol",
-					DBUS_TYPE_STRING, &protocol);
+	dbus_message_iter_append_basic(iter, DBUS_TYPE_STRING, &type);
 
-	readonly = tag->readonly;
-	near_dbus_dict_append_basic(&dict, "ReadOnly",
-					DBUS_TYPE_BOOLEAN, &readonly);
-
-	near_dbus_dict_append_array(&dict, "Records",
-				DBUS_TYPE_OBJECT_PATH, append_records, tag);
-
-	near_dbus_dict_close(&array, &dict);
-
-	return reply;
+	return TRUE;
 }
 
-static DBusMessage *set_property(DBusConnection *conn,
-					DBusMessage *msg, void *data)
+static gboolean property_get_protocol(const GDBusPropertyTable *property,
+					DBusMessageIter *iter, void *user_data)
 {
-	DBG("conn %p", conn);
+	struct near_tag *tag = user_data;
+	const char *protocol;
+
+	protocol = protocol_string(tag);
+	if (!protocol)
+		return FALSE;
+
+	dbus_message_iter_append_basic(iter, DBUS_TYPE_STRING, &protocol);
+
+	return TRUE;
+}
+
+static gboolean property_get_readonly(const GDBusPropertyTable *property,
+					DBusMessageIter *iter, void *user_data)
+{
+	struct near_tag *tag = user_data;
+	dbus_bool_t readonly;
+
+	readonly = tag->readonly;
+
+	dbus_message_iter_append_basic(iter, DBUS_TYPE_BOOLEAN, &readonly);
+
+	return TRUE;
+}
+
+static gboolean property_get_adapter(const GDBusPropertyTable *property,
+					DBusMessageIter *iter, void *user_data)
+{
+	struct near_tag *tag = user_data;
+	struct near_adapter *adapter;
+	const char *path;
+
+	adapter = __near_adapter_get(tag->adapter_idx);
+	if (!adapter)
+		return FALSE;
+
+	path = __near_adapter_get_path(adapter);
+	if (!path)
+		return FALSE;
+
+	dbus_message_iter_append_basic(iter, DBUS_TYPE_OBJECT_PATH, &path);
+
+	return TRUE;
 
-	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 }
 
 static void tag_read_cb(uint32_t adapter_idx, uint32_t target_idx, int status)
@@ -239,12 +254,12 @@ static void tag_read_cb(uint32_t adapter_idx, uint32_t target_idx, int status)
 	if (!tag)
 		return;
 
-	dbus_message_unref(tag->write_msg);
-	tag->write_msg = NULL;
+	if (tag->write_msg) {
+		dbus_message_unref(tag->write_msg);
+		tag->write_msg = NULL;
+	}
 
 	__near_adapter_start_check_presence(adapter_idx, target_idx);
-
-	__near_adapter_tags_changed(adapter_idx);
 }
 
 static void write_cb(uint32_t adapter_idx, uint32_t target_idx, int status)
@@ -272,7 +287,6 @@ static void write_cb(uint32_t adapter_idx, uint32_t target_idx, int status)
 	}
 
 	near_ndef_records_free(tag->records);
-	tag->n_records = 0;
 	tag->records = NULL;
 	g_free(tag->data);
 	tag->data = NULL;
@@ -448,24 +462,20 @@ fail:
 }
 
 static const GDBusMethodTable tag_methods[] = {
-	{ GDBUS_METHOD("GetProperties",
-				NULL, GDBUS_ARGS({"properties", "a{sv}"}),
-				get_properties) },
-	{ GDBUS_METHOD("SetProperty",
-				GDBUS_ARGS({"name", "s"}, {"value", "v"}),
-				NULL, set_property) },
 	{ GDBUS_ASYNC_METHOD("Write", GDBUS_ARGS({"attributes", "a{sv}"}),
 							NULL, write_ndef) },
 	{ },
 };
 
-static const GDBusSignalTable tag_signals[] = {
-	{ GDBUS_SIGNAL("PropertyChanged",
-				GDBUS_ARGS({"name", "s"}, {"value", "v"})) },
+static const GDBusPropertyTable tag_properties[] = {
+	{ "Type", "s", property_get_type },
+	{ "Protocol", "s", property_get_protocol },
+	{ "ReadOnly", "b", property_get_readonly },
+	{ "Adapter", "o", property_get_adapter },
+
 	{ }
 };
 
-
 void __near_tag_append_records(struct near_tag *tag, DBusMessageIter *iter)
 {
 	GList *list;
@@ -595,7 +605,7 @@ static int tag_initialize(struct near_tag *tag,
 	tag->adapter_idx = adapter_idx;
 	tag->target_idx = target_idx;
 	tag->protocol = protocols;
-	tag->n_records = 0;
+	tag->next_record = 0;
 	tag->readonly = false;
 
 	if (nfcid_len <= NFC_MAX_NFCID1_LEN) {
@@ -644,8 +654,8 @@ struct near_tag *__near_tag_add(uint32_t adapter_idx, uint32_t target_idx,
 
 	g_dbus_register_interface(connection, tag->path,
 					NFC_TAG_INTERFACE,
-					tag_methods, tag_signals,
-							NULL, tag, NULL);
+					tag_methods, NULL,
+				        tag_properties, tag, NULL);
 
 	return tag;
 }
@@ -656,12 +666,6 @@ void __near_tag_remove(struct near_tag *tag)
 
 	DBG("path %s", tag->path);
 
-	if (!g_hash_table_lookup(tag_hash, tag->path))
-		return;
-
-	g_dbus_unregister_interface(connection, tag->path,
-						NFC_TAG_INTERFACE);
-
 	g_hash_table_remove(tag_hash, path);
 }
 
@@ -769,14 +773,14 @@ int near_tag_add_records(struct near_tag *tag, GList *records,
 
 		path = g_strdup_printf("%s/nfc%d/tag%d/record%d",
 					NFC_PATH, tag->adapter_idx,
-					tag->target_idx, tag->n_records);
+					tag->target_idx, tag->next_record);
 
 		if (!path)
 			continue;
 
 		__near_ndef_record_register(record, path);
 
-		tag->n_records++;
+		tag->next_record++;
 		tag->records = g_list_append(tag->records, record);
 	}
 
@@ -1058,6 +1062,9 @@ static void free_tag(gpointer data)
 
 	near_ndef_records_free(tag->records);
 
+	g_dbus_unregister_interface(connection, tag->path,
+						NFC_TAG_INTERFACE);
+
 	g_free(tag->path);
 	g_free(tag->data);
 	g_free(tag);
